
# q-expansion of newform 1872.2.a.g, downloaded from the LMFDB on 01 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1872
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [0], [-2], [2], [4], [-1], [0], [6], [-4], [-8], [2], [6], [6], [8], [-8], [12], [-4], [10], [2], [16], [14], [4], [12], [-6], [-10], [-16], [-4], [20], [10], [4], [8], [-4], [-18], [20], [-6], [-18], [2], [10], [0], [16], [-12], [-18], [4], [2], [-18], [-8], [-8], [-6], [-12], [-30], [20], [24], [6], [0], [12], [0], [0], [-22], [-6], [-10], [20], [2], [-2], [12], [26], [6], [10], [-34], [24], [22], [14], [0], [-8], [-10], [-10], [0], [-12], [-10], [-6], [-26], [-36], [10], [0], [-2], [4], [-36], [26], [-2], [30], [-22], [8], [0], [-38], [16], [-22], [-32], [-10], [-36], [12], [2], [20], [18], [20], [-4], [-36], [-30], [-28], [-38], [40], [10], [-24], [-34], [14], [-34], [14], [-40], [10], [-8], [0], [48], [14], [22], [-12], [4], [26], [-12], [-22], [-28], [8], [-22], [54], [40], [12], [-18], [30], [-30], [-14], [22], [-12], [36], [-50], [34], [12], [12], [46], [-16], [-26], [-28], [-8], [-8], [6], [36], [20], [-44], [-12], [-12], [-12], [30], [30], [26], [-36], [28], [58], [-48], [18], [32], [48], [38], [-10], [16], [36], [-18], [-20], [46], [-60], [-40], [22], [-22], [44], [-34], [-6], [-44], [46], [-38], [16], [-52], [22], [-42], [-10], [-24], [-14], [12], [44], [-14], [4], [-36], [2], [34], [10], [28], [30], [40], [2], [-14], [52], [-12], [-22], [-36], [18], [-60], [14], [60], [52], [28], [24], [-62], [68], [-8], [40], [-6], [-38], [-14], [-54], [-46], [-24], [-58], [-56], [-24], [56], [-60], [22], [-28], [6], [32], [-68], [32], [-50], [-22], [-60], [-4], [-36], [-52], [12], [-10], [-62], [60], [-62], [28], [-2], [36], [50], [-26], [-8], [-10], [56], [-44], [42], [-26], [-12], [34], [8], [48], [-46], [-2], [18], [-16], [-58], [-2], [6], [-48], [-14], [-22], [30], [4], [62], [-18], [-4], [6], [-38], [0], [44], [-30], [-52], [-10], [6], [16], [74], [10], [46], [-44], [12], [8], [-54], [44], [-14], [36], [48], [28], [40], [-2], [74], [-50], [-60], [-16], [44], [-34], [-68], [-78], [16], [6], [-24], [22], [12], [-16], [-64], [38], [-60], [32], [-10], [64], [-32], [-14], [60], [70], [-86], [-38], [-22], [42], [40], [64], [10], [40], [-20], [36], [46], [20], [-10], [22], [-42], [-44], [-18], [12], [-70], [92], [46], [76], [-38], [82], [32], [-12], [-22], [-2], [-30], [88], [-6], [-12], [24], [36], [-12], [-72], [46], [-48], [-40], [-4], [40], [70], [-18], [-82], [78], [12], [-88], [-16], [-60], [60], [66], [36], [24], [-70], [-80], [-14], [-54], [22], [18], [26], [26], [-72], [54], [38], [-2], [8], [6], [2], [-60], [-8], [24], [34], [-66], [-80], [48], [6], [22], [-104], [-94], [50], [78], [28], [10], [42], [86], [-36], [34], [84], [-40], [-64], [-10], [-36], [-72], [-4], [-102], [-84], [40], [-26], [48], [60], [-46], [50], [32], [-6], [74], [28]]
